<?php
// $Id: pifr_client.admin.inc,v 1.10 2009/07/17 19:44:45 boombatower Exp $
/**
 * @file
 * Provide general administration functionality.
 *
 * Copyright 2008-2009 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Dispaly report value on general dashboard.
 */
function pifr_client_admin_dashboard_form(&$form, $form_state) {
  module_load_include('cron.inc', 'pifr_client');
  $form['status']['client'] = array(
    '#type' => 'item',
    '#title' => t('Client'),
    '#value' => (pifr_client_cron_is_busy() ?
                 t('testing [@count assertions]', array('@count' => pifr_client_admin_assertion_count())) :
                 t('idle')) . '<br />',
  );
  $form['status']['toggle_reset'] = array(
    '#type' => 'submit',
    '#value' => t('Toggle reset'),
    '#submit' => array('pifr_client_admin_dashboard_form_toggle_submit'),
    '#weight' => 1001
  );
}

/**
 * Get the number of assertions that have been made during the current test
 * suite run.
 *
 * @return integer Number of assertions.
 */
function pifr_client_admin_assertion_count() {
  require_once drupal_get_path('module', 'pifr_client') . '/review/db/db.inc';
  require_once drupal_get_path('module', 'pifr_client') . '/review/db/' . PIFR_CLIENT_DB . '.inc';
  $database = 'PIFRClientReviewDB_' . PIFR_CLIENT_DB;
  $database = new $database;

  $assertions = $database->getAssertions();
  $count = 0;
  while ($assertion = db_fetch_array($assertions)) {
    $count++;
  }
  return $count;
}

/**
 * Reset variables.
 */
function pifr_client_admin_dashboard_form_toggle_submit($form, &$form_state) {
  module_load_include('cron.inc', 'pifr_client');
  pifr_client_cron_reset();
}

/**
 * Add report setting to configuration form.
 */
function pifr_client_admin_configuration_form(&$form, $form_state) {
  $form['#validate'][] = 'pifr_client_admin_configuration_form_validate';
  $form['client'] = array(
    '#type' => 'fieldset',
    '#title' => t('Client'),
    '#weight' => -9
  );
  $form['client']['pifr_client_key'] = array(
    '#type' => 'textfield',
    '#title' => t('Key'),
    '#description' => t('The key assigned to client.'),
    '#default_value' => PIFR_CLIENT_KEY,
    '#required' => TRUE,
  );
  $form['client']['pifr_client_server'] = array(
    '#type' => 'textfield',
    '#title' => t('Server URL'),
    '#description' => t('The URL of the server to retrive patches from, include ending forward slash.'),
    '#default_value' => PIFR_CLIENT_SERVER,
    '#required' => TRUE,
  );
  $form['client']['pifr_client_db'] = array(
    '#type' => 'radios',
    '#title' => t('Database type'),
    '#options' => array(
      'mysql' => t('MySQL'),
      'pgsql' => t('PostgreSQL'),
      'sqlite' => t('SQLite'),
    ),
    '#default_value' => PIFR_CLIENT_DB,
    '#required' => TRUE,
  );
  $form['client']['pifr_client_concurrency'] = array(
    '#type' => 'select',
    '#title' => t('Concurrency'),
    '#description' => t('The number of concurrent process to use when running tests. (see run-tests.sh)'),
    '#options' => drupal_map_assoc(array(1, 2, 3, 4, 5, 6, 7, 8)),
    '#default_value' => PIFR_CLIENT_CONCURRENCY,
  );
  $form['client']['pifr_client_timeout'] = array(
    '#type' => 'textfield',
    '#title' => t('Testing timeout'),
    '#description' => t('The maximum time, in minutes, allowed for tests to complete.'),
    '#size' => 4,
    '#default_value' => PIFR_CLIENT_TIMEOUT,
    '#required' => TRUE,
  );
}

/**
 * Validate client settings.
 */
function pifr_client_admin_configuration_form_validate($form, &$form_state) {
  if (drupal_strlen($form_state['values']['pifr_client_key']) != 32) {
    form_set_error('pifr_client_key', t('Key must be 32 characters long.'));
  }

  if (!$form_state['values']['pifr_client_server'] || !@parse_url($form_state['values']['pifr_client_server'])) {
    form_set_error('pifr_client_server', t('Server URL must be a complete URL, schema included.'));
  }
  else if (drupal_substr($form_state['values']['pifr_client_server'],
                          drupal_strlen($form_state['values']['pifr_client_server']) - 1) != '/') {
    form_set_error('pifr_client_server', t('Server URL must end with a slash.'));
  }
  else {
    $result = xmlrpc($form_state['values']['pifr_client_server'] . '/xmlrpc.php', 'system.listMethods');
    if ($result !== FALSE) {
      // Check for PIFR server methods.
      if (!is_array($result) || !in_array('pifr.next', $result)) {
        form_set_error('pifr_client_server', t('The server does not appear to be running PIFR server 2.x.'));
      }
    }
    else {
      form_set_error('pifr_client_server', t('URL does not point to a valid server XML-RPC root.'));
    }
  }

  if (!extension_loaded('pdo_' . $form_state['values']['pifr_client_db'])) {
    form_set_error('pifr_client_db', t('The @db PHP PDO extension must be installed.',
                                       array('@db' => $form_state['values']['pifr_client_db'])));
  }
}

/**
 * Test the server configuration and display the results.
 *
 * @return string HTML output.
 */
function pifr_client_admin_check_configuration() {
  drupal_add_css(drupal_get_path('module', 'pifr_client') . '/pifr_client.css');
  $out = '';

  // Test the firewall rules to make sure that the server can
  // communicate with itself ($base_url), drupal.org, testing.drupal.org,
  // and cvs.drupal.org as required.
  $out .= '<h3>' . t('Firewall/connection') . '</h3>';
  $out .= '<div class="pifr-client-section">';

  $connections = array(
    'required' => array(
      'title' => t('Required'),
      'description' => t('All of the following connection tests must pass in order for your slave to ' .
                         'be able to properly communicate with the testing server.'),
      'tests' => array(
        array(
          'hostname' => 'drupal.org',
          'port' => 80,
        ),
        array(
          'hostname' => 'testing.drupal.org',
          'port' => 80,
        ),
        array(
          'hostname' => 'cvs.drupal.org',
          'port' => 2401,
        ),
        array(
          'hostname' => parse_url(url('', array('absolute' => TRUE)), PHP_URL_HOST),
          'port' => 80,
        ),
      ),
    ),
    'recommended' => array(
      'title' => t('Recommended'),
      'description' => t('Because of the possibility that insecure code in a patch being tested on this ' .
                         'machine could be executed, it is recommended that outgoing connections from ' .
                         'this server be blocked except when required. <em>Therefore, for the most secure ' .
                         'configuration, all of the following tests should fail.</em>'),
      'tests' => array(
        array(
          'hostname' => 'www.google.com',
          'port' => 80,
        ),
        array(
          'hostname' => 'mail.google.com',
          'port' => 25,
        ),
      ),
    ),
  );

  foreach ($connections as $key => $connection) {
    $out .= '<h4 class=\"pifr-test-connection-title\">' . $connection['title'] . '</h4>';
    $out .= '<div class=\"pifr-test-connection-description\">' . $connection['description'] . '</div>';
    $header = array(t('Host name'), t('Port'), t('Result'), t('Error message'));
    $rows = array();
    foreach ($connection['tests'] as $test) {
      $result = pifr_client_admin_test_connection($test['hostname'], $test['port']);
      if (!empty($result['error_number'])) {
        $rows[] = array($test['hostname'], $test['port'], t('Failed'), $result['error_message']);
      }
      else {
        $rows[] = array($test['hostname'], $test['port'], t('Passed'), '');
      }
    }
    $out .= theme('table', $header, $rows);
  }
  $out .= '</div>';

  // Check for required command line tools.
  $commands = array(
    array(
      'title' => 'patch',
      'cmd' => 'patch -v',
    ),
    array(
      'title' => 'cvs',
      'cmd' => 'cvs -v',
    ),
    array(
      'title' => 'curl',
      'cmd' => 'curl --version',
    ),
    array(
      'title' => 'php (cli)',
      'cmd' => 'php -v',
    ),
    array(
      'title' => 'grep',
      'cmd' => 'grep --version',
    ),
    array(
      'title' => 'mysql',
      'cmd' => 'mysql --version',
    ),
  );

  $out .= '<h3>' . t('Command line tools') . '</h3>';
  $out .= '<div class="pifr-client-section">';
  $header = array(t('Name'), t('Shell command'), t('Output'));
  $rows = array();
  foreach ($commands as $command) {
    $cmd_output = shell_exec($command['cmd']);
    $rows[] = array($command['title'], check_plain($command['cmd']), '<code>' . check_plain($cmd_output) . '</code>');
  }

  $out .= theme('table', $header, $rows);
  $out .= '</div>';

  // Check for required PHP Extensions.
  $out .= '<h3>' . t('PHP Extensions') . '</h3>';
  $out .= '<div class="pifr-client-section">';
  $header = array(t('Extension name'), t('Minimum required version'), t('Version found'), t('Result'));
  $rows = array();

  // Check for CURL.
  $min_version = '7';
  $found_version = '';
  $result = FALSE;
  if (function_exists('curl_version')) {
    $info = curl_version();
    $found_version = $info['version'];
    if (version_compare($min_version, $found_version) <= 0) {
      $result = TRUE;
    }
  }
  $rows[] = array(l('CURL', 'http://www.php.net/manual/en/book.curl.php', array('absolute' => TRUE)),
                   $min_version, $found_version, $result ? t('Passed') : t('Failed'));

  // Check for presence of gd library.
  $min_version = '2';
  $found_version = '2';
  $result = FALSE;
  if (function_exists('imagegd2')) {
    $result = TRUE;
  }
  $rows[] = array(l('gd', 'http://www.php.net/manual/en/ref.image.php', array('absolute' => TRUE)),
                   $min_version, $found_version, $result ? t('Passed') : t('Failed'));

  // Check for presence of mbstring library.
  $min_version = UNICODE_MULTIBYTE;
  $found_version = -1;
  $result = FALSE;
  $found_version = $GLOBALS['multibyte'];
  if ($min_version <= $found_version) {
    $result = TRUE;
  }
  $rows[] = array(l('mbstring', 'http://www.php.net/manual/en/book.mbstring.php', array('absolute' => TRUE)),
                   $min_version, $found_version, $result ? t('Passed') : t('Failed'));

  // Check for PDO.
  $min_version = 1;
  $found_version = 0;
  $result = FALSE;
  $found_version = phpversion('pdo');
  if ($min_version <= $found_version) {
    $result = TRUE;
  }
  $rows[] = array(l('pdo', 'http://www.php.net/manual/en/book.pdo.php', array('absolute' => TRUE)),
                   $min_version, $found_version, $result ? t('Passed') : t('Failed'));

  $out .= theme('table', $header, $rows);
  $out .= '</div>';

  // Check for recommended PHP configuration.
  $out .= '<h3>' . t('PHP Configuration') . '</h3>';
  $out .= '<div class="pifr-client-section">';
  $header = array(t('Setting'), t('Recommended value'), t('Value found'), t('Result'));
  $rows = array();

  // Test memory limit (Apache).
  $result = FALSE;
  $memory_limit = ini_get('memory_limit');
  if (parse_size($memory_limit) >= parse_size(PIFR_RECOMMENDED_MEMORY)) {
    $result = TRUE;
  }
  $rows[] = array(l('memory_limit', 'http://www.php.net/manual/en/ini.core.php#ini.memory-limit', array('absolute' => TRUE)),
                   PIFR_RECOMMENDED_MEMORY, $memory_limit, $result ? t('Passed') : t('Failed'));

  // Test memory limit (cli).
  $result = FALSE;
  $memory_limit = `php -r "echo ini_get('memory_limit');"`;
  if (parse_size($memory_limit) >= parse_size(PIFR_RECOMMENDED_MEMORY)) {
    $result = TRUE;
  }
  $rows[] = array(l('memory_limit (cli)', 'http://www.php.net/manual/en/ini.core.php#ini.memory-limit',
                   array('absolute' => TRUE)), PIFR_RECOMMENDED_MEMORY, $memory_limit, $result ? t('Passed') : t('Failed'));

  $out .= theme('table', $header, $rows);
  $out .= '</div>';

  return $out;
}

/**
 * Test connectivity to a host via a certain port.
 *
 * @param string $hostname A host name or IP address to connect to.
 * @param integer $port The port number to connect to.
 * @param interger $timeout The number of seconds to wait before timing out on
 *   the connection attempt.
 * @return
 *   A keyed array containing the error number and error string as returned
 *   by the PHP function fsockopen().
 */
function pifr_client_admin_test_connection($hostname, $port, $timeout = 2) {
  $error_number = NULL;
  $error_string = '';
  $fp = @fsockopen($hostname, $port, $error_number, $error_message, $timeout);
  if ($fp) {
    fclose($fp);
  }
  return array('error_number' => $error_number, 'error_message' => $error_message);
}
