<?php
// $Id: pifr_client.xmlrpc.inc,v 1.5 2009/07/09 01:39:47 boombatower Exp $
/**
 * @file
 * Provide XML-RPC callbacks and invoke functions.
 *
 * Copyright 2008-2009 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Retreive the PIFR version string.
 *
 * @return string PIFR version string.
 */
function pifr_client_xmlrpc_version() {
  $info = parse_ini_file(drupal_get_path('module', 'pifr') . '/pifr.info');
  return (string) (isset($info['version']) ? $info['version'] : 'unknown');
}

/**
 * Request the next test from the server.
 *
 * @return mixed Test array of FALSE.
 */
function pifr_client_xmlrpc_request_next() {
  $response = xmlrpc(PIFR_CLIENT_SERVER . 'xmlrpc.php', 'pifr.next', PIFR_CLIENT_KEY);

  if ($response === FALSE) {
    watchdog('pifr_client', 'Failed to request next test: @message', array('@message' => xmlrpc_error_msg()), WATCHDOG_ERROR);
    return FALSE;
  }
  else if (isset($response['response'])) {
    watchdog('pifr_client', 'When requesting next test, server responded: @message',
                      array('@message' => pifr_response_code($response['response'])), WATCHDOG_NOTICE);
    return FALSE;
  }
  else if (!$response) {
    // Empty array, no tests.
    return FALSE;
  }

  watchdog('pifr_client', 'Test request successfull, received: (t: @test_id).', array('@test_id' => $response['test_id']));
  watchdog('pifr_client', 'XML-RPC pifr.next() response: '. var_export($response, TRUE));

  return $response;
}

function pifr_client_xmlrpc_send_result(array $result) {
  $response = xmlrpc(PIFR_CLIENT_SERVER . 'xmlrpc.php', 'pifr.result', PIFR_CLIENT_KEY, $result);

  if ($response === FALSE) {
    watchdog('pifr_client', 'Failed to send result: @message', array('@message' => xmlrpc_error_msg()), WATCHDOG_ERROR);
    return FALSE;
  }
  else if ($response != PIFR_RESPONSE_ACCEPTED) {
    watchdog('pifr_client', 'When sending result, server responded: @message',
                      array('@message' => pifr_response_code($response)), WATCHDOG_NOTICE);
    return FALSE;
  }

  watchdog('pifr_client', 'Test results successfull sent: (t: @test_id).', array('@test_id' => $result['test_id']));
  watchdog('pifr_client', 'XML-RPC pifr.result() response: '. var_export($response, TRUE));

  return TRUE;
}
