<?php
// $Id: pifr_server.branch.inc,v 1.12 2009/03/13 03:07:27 boombatower Exp $
/**
 * @file
 * Provide branch functions.
 *
 * Copyright 2008-2009 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

function pifr_server_branch_get($branch_id) {
  return db_fetch_array(db_query('SELECT *
                                   FROM {pifr_branch}
                                   WHERE branch_id = %d', $branch_id));
}

function pifr_server_branch_find($branch_nid, $project_id) {
  return db_fetch_array(db_query('SELECT *
                                   FROM {pifr_branch}
                                   WHERE branch_nid = %d
                                   AND project_id = %d', $branch_nid, $project_id));
}

function pifr_server_branch_get_test($test_id) {
  return db_fetch_array(db_query('SELECT *
                                   FROM {pifr_branch}
                                   WHERE test_id = %d', $test_id));
}

function pifr_server_branch_save(array $branch, $check = TRUE) {
  if ($check) {
    // Check if branch already exists.
    $branch_record = pifr_server_branch_find($branch['branch_nid'], $branch['project_id']);
    $branch['branch_id'] = ($branch_record ? $branch_record['branch_id'] : NULL);
    $branch['test_id'] = ($branch_record ? $branch_record['test_id'] : NULL);
  }

  if (isset($branch['branch_id'])) {
    // Existing project, update record.
    db_query("UPDATE {pifr_branch}
              SET branch_nid = %d,
                  project_id = %d,
                  core_branch_id = %d,
                  identifier = '%s',
                  test_id = %d
              WHERE branch_id = %d", $branch['branch_nid'], $branch['project_id'], $branch['core_branch_id'],
                                     $branch['identifier'], $branch['test_id'], $branch['branch_id']);
  }
  else {
    // Insert test record.
    $test = array(
      'type' => PIFR_SERVER_TEST_TYPE_BRANCH,
      'status' => PIFR_SERVER_TEST_STATUS_STUB,
      'client_id' => 0,
      'last_received' => time(),
      'last_requested' => 0,
      'last_tested' => 0,
      'test_count' => 0
    );
    $test = pifr_server_test_save($test);
    $branch['test_id'] = $test['test_id'];

    // New branch, insert record.
    db_query("INSERT INTO {pifr_branch} (branch_nid, project_id, core_branch_id, identifier, test_id)
              VALUES (%d, %d, %d, '%s', %d)",
              $branch['branch_nid'], $branch['project_id'], $branch['core_branch_id'],
              $branch['identifier'], $branch['test_id']);
    $branch['branch_id'] = db_last_insert_id('pifr_branch', 'branch_id');
  }
  return $branch;
}

class PIFRTestBranch extends PIFRTest {

  protected $branch;

  public function __construct(array $test) {
    parent::__construct($test);
    $this->branch = pifr_server_branch_get_test($test['test_id']);
  }

  public function getType() {
    return t('Branch');
  }

  public function getTitle() {
    return check_plain($this->branch['identifier']);
  }

  public function getInfo() {
    return $this->branch;
  }

  public function getInfoTable() {
    $rows = array();

    $rows['identifier'] = array(t('Identifier'), check_plain($this->branch['identifier']));

    $project = pifr_server_project_get($this->branch['project_id']);
    $client = pifr_server_client_get($project['client_id']);
    $rows['project'] = array(t('Project'), l($project['name'], "{$client['url']}node/{$project['project_nid']}"));

    return $this->formatTable($rows);
  }
}
