<?php
// $Id: pifr_server.client.inc,v 1.30 2009/06/18 22:31:12 boombatower Exp $
/**
 * @file
 * Provide client functions.
 *
 * Copyright 2008-2009 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Get client by ID.
 *
 * @param integer $client_id Client ID.
 * @return mixed Client or FALSE.
 */
function pifr_server_client_get($client_id) {
  $result = db_query('SELECT *
                      FROM {pifr_client}
                      WHERE client_id = %d', $client_id);
  return db_fetch_array($result);
}

/**
 * Get client by client key.
 *
 * @param string $client_key Client key.
 * @return mixed Client or FALSE.
 */
function pifr_server_client_get_key($client_key) {
  $result = db_query("SELECT *
                      FROM {pifr_client}
                      WHERE client_key = '%s'", $client_key);
  return db_fetch_array($result);
}

function pifr_server_client_get_test($test_id) {
  $result = db_query("SELECT *
                      FROM {pifr_client}
                      WHERE test_id = %d", $test_id);
  return db_fetch_array($result);
}

/**
 * Get clients by type.
 *
 * @param interger $client_type Client type.
 * @return array List of clients.
 */
function pifr_server_client_get_type($client_type) {
  $result = db_query('SELECT c.*, t.last_requested AS last_active
                      FROM {pifr_client} c
                      LEFT JOIN {pifr_test} t
                        ON (
                          c.client_id = t.client_id
                          AND t.last_requested = (SELECT MAX(last_requested) FROM {pifr_test} tt WHERE tt.client_id = c.client_id)
                        )
                      WHERE c.type = %d
                      ORDER BY last_active DESC', $client_type);

  $clients = array();
  while ($client = db_fetch_array($result)) {
    $clients[] = $client;
  }
  return $clients;
}

/**
 * Get client by key and type (client must be enabled or testing).
 *
 * @param string $client_key Client key.
 * @param integer $type Client type ID.
 * @return mixed Client object or FALSE.
 */
function pifr_server_client_check_key($client_key, $type) {
  $client = pifr_server_client_get_key($client_key);
  if ($client && $client['type'] == $type &&
      ($client['status'] == PIFR_SERVER_CLIENT_STATUS_ENABLED || $client['status'] == PIFR_SERVER_CLIENT_STATUS_TESTING)) {
    return $client;
  }
  return FALSE;
}

/**
 * Get all client objects.
 *
 * @param interger $uid User ID to which clients must belong.
 * @return array List of clients.
 */
function pifr_server_client_get_all($uid = NULL) {
  if ($uid) {
    $result = db_query('SELECT *
                        FROM {pifr_client}
                        WHERE uid = %d', $uid);
  }
  else {
    $result = db_query('SELECT *
                        FROM {pifr_client}');
  }

  $clients = array();
  while ($client = db_fetch_array($result)) {
    $clients[] = $client;
  }
  return $clients;
}

/**
 * Initialize the confirmation process.
 *
 * @param array $client Client.
 */
function pifr_server_client_confirmation_init(array $client) {
  if ($client['type'] == PIFR_SERVER_CLIENT_TYPE_PROJECT) {
    $client['status'] = PIFR_SERVER_CLIENT_STATUS_ENABLED;
    pifr_server_client_save($client);
    return;
  }

  $client['status'] = PIFR_SERVER_CLIENT_STATUS_TESTING;
  pifr_server_client_save($client);

  $test = pifr_server_test_get($client['test_id']);
  pifr_server_test_queue($test);

  // Check client versin number.
  if (!pifr_server_client_confirmation_check_version($client)) {
    pifr_server_client_confirmation_fail($client);

    // TODO Store message that explains version issue.
  }
}

function pifr_server_client_confirmation_check_version(array $client) {
  $info = parse_ini_file(drupal_get_path('module', 'pifr') . '/pifr.info');
  $version = (isset($info['version']) ? $info['version'] : 'unknown');

  $return = xmlrpc($client['url'] . 'xmlrpc.php', 'pifr.version');
  return ($return === FALSE ? FALSE : $return == $version);
}

function pifr_server_client_confirmation_next($client) {
  $last = pifr_server_client_confirmation_last($client);

  // Get the next confirmation test.
  return pifr_server_client_confirmation_get_test($client, $last + 1);
}

function pifr_server_client_confirmation_last($client) {
  $results = pifr_server_result_get_all($client['test_id']);

  // Determine the last test type run, PIFR_SERVER_TEST_TYPE_*.
  if (!$results) {
    $last = 1; // Assume setup passes, PIFR_SERVER_TEST_TYPE_SETUP (1).
  }
  else {
    $last = array_pop($results);
    $last = $last['type'];

    if ($last == PIFR_SERVER_TEST_RESULT_SYNTAX) {
      // TODO Skip.
      $last++;
    }
  }
  return $last;
}

/**
 * Get a test that will ensure that the specified result code is triggered.
 *
 * @param object Client.
 * @param integer $type Result code.
 * @return unknown_type
 */
function pifr_server_client_confirmation_get_test(array $client, $type) {
  switch ($type) {
    case PIFR_SERVER_TEST_RESULT_SETUP:
      break;
    case PIFR_SERVER_TEST_RESULT_FETCH:
      return pifr_server_client_confirmation_get_test_array($client, array(
        'file_url' => 'http://example.com/pifr_server/some.patch',
      ));
    case PIFR_SERVER_TEST_RESULT_CVS:
      return pifr_server_client_confirmation_get_test_array($client, array(
        'core' => array(
          'repository' => array(
            'type' => 'cvs',
            'url' => 'example.com/cvs',
          ),
        ),
      ));
    case PIFR_SERVER_TEST_RESULT_PATCH:
      return pifr_server_client_confirmation_get_test_array($client, array(
        'file_url' => url(file_directory_path() . '/pifr_server/patch.patch', array('absolute' => TRUE)),
      ));
    case PIFR_SERVER_TEST_RESULT_APPLY:
      return pifr_server_client_confirmation_get_test_array($client, array(
        'file_url' => url(file_directory_path() . '/pifr_server/apply.patch', array('absolute' => TRUE)),
      ));
    case PIFR_SERVER_TEST_RESULT_SYNTAX:
      return pifr_server_client_confirmation_get_test_array($client, array(
        'file_url' => url(file_directory_path() . '/pifr_server/syntax.patch', array('absolute' => TRUE)),
      ));
    case PIFR_SERVER_TEST_RESULT_FORMAT:
      // TODO Not implemented.
      break;
    case PIFR_SERVER_TEST_RESULT_INSTALL:
      return pifr_server_client_confirmation_get_test_array($client, array(
        'file_url' => url(file_directory_path() . '/pifr_server/install.patch', array('absolute' => TRUE)),
      ));
    case PIFR_SERVER_TEST_RESULT_TEST:
      return pifr_server_client_confirmation_get_test_array($client, array(
        'file_url' => url(file_directory_path() . '/pifr_server/test.patch', array('absolute' => TRUE)),
      ));
    case PIFR_SERVER_TEST_RESULT_FAIL:
      return pifr_server_client_confirmation_get_test_array($client, array(
        'file_url' => url(file_directory_path() . '/pifr_server/fail.patch', array('absolute' => TRUE)),
      ));
    case PIFR_SERVER_TEST_RESULT_PASS:
      return pifr_server_client_confirmation_get_test_array($client, array(
        'file_url' => url(file_directory_path() . '/pifr_server/pass.patch', array('absolute' => TRUE)),
      ));
  }
}

function pifr_server_client_confirmation_get_test_array($client, array $changes) {
  $default = array(
    'test_id' => $client['test_id'],
    'core' => array(
      'repository' => array(
        'type' => 'cvs',
        'url' => ':pserver:anonymous:anonymous@cvs.drupal.org:/cvs/drupal/drupal',
      ),
      'branch_identifier' => 'HEAD',
    ),
  );
  return array_merge($default, $changes);
}

function pifr_server_client_confirmation_check(array $client, array $result) {
  if ($result['type'] != $result['code']) {
    pifr_server_client_confirmation_fail($client);
    return;
  }

  if ($result['type'] == PIFR_SERVER_TEST_RESULT_PASS) {
    pifr_server_client_confirmation_pass($client);
  }
}

function pifr_server_client_confirmation_pass(array $client) {
  $client['status'] = PIFR_SERVER_CLIENT_STATUS_ENABLED;
  pifr_server_client_save($client);

  // TODO More detailed log message.
  pifr_server_log_event($client['test_id'], t('Passed confirmation tests.'));
}

function pifr_server_client_confirmation_fail(array $client) {
  $client['status'] = PIFR_SERVER_CLIENT_STATUS_FAIL;
  pifr_server_client_save($client);

  // TODO More detailed log message.
  pifr_server_log_event($client['test_id'], t('Failed confirmation tests.'));
}

/**
 * Save client information.
 *
 * @param array $client Associative array of client information.
 * @return array Updated client array.
 */
function pifr_server_client_save(array $client) {
  if (isset($client['client_id'])) {
    // Existing client, update record.
    db_query("UPDATE {pifr_client}
              SET client_key = '%s',
                type = %d,
                environment_id = %d,
                uid = %d,
                url = '%s',
                status = %d,
                test_id = %d
              WHERE client_id = %d",
              $client['client_key'], $client['type'], $client['environment_id'], $client['uid'],
              $client['url'], $client['status'], $client['test_id'], $client['client_id']);
  }
  else {
    // Insert test record.
    $test = array(
      'type' => PIFR_SERVER_TEST_TYPE_CLIENT,
      'status' => PIFR_SERVER_TEST_STATUS_QUEUED,
      'client_id' => 0,
      'last_received' => 0,
      'last_requested' => 0,
      'last_tested' => 0,
      'test_count' => 0
    );
    $test = pifr_server_test_save($test);

    // New client, insert record.
    db_query("INSERT INTO {pifr_client} (client_key, type, environment_id, uid, url, status, test_id)
              VALUES ('%s', %d, %d, %d, '%s', %d, %d)",
              $client['client_key'],  $client['type'], $client['environment_id'], $client['uid'],
              $client['url'], $client['status'], $test['test_id']);
    $client['client_id'] = db_last_insert_id('pifr_client', 'client_id');
  }
  return $client;
}

function pifr_server_client_remove($client_id) {
  db_query('DELETE FROM {pifr_client}
            WHERE client_id = %d', $client_id);
}

/**
 * Get the client type from the code.
 *
 * @param integer $client_type Client type code.
 * @return string Representation of client type.
 */
function pifr_server_client_type($client_type) {
  switch ($client_type) {
    case PIFR_SERVER_CLIENT_TYPE_PROJECT:
      return t('Project');
    case PIFR_SERVER_CLIENT_TYPE_TEST:
      return t('Test');
    default:
      return 'Unknown';
  }
}

/**
 * Get the client status from the code.
 *
 * @param integer $client_status Client status code.
 * @return string Representation of client status.
 */
function pifr_server_client_status($client_status) {
  switch ($client_status) {
    case PIFR_SERVER_CLIENT_STATUS_DISABLED:
      return t('Disabled');
    case PIFR_SERVER_CLIENT_STATUS_TESTING:
      return t('Testing');
    case PIFR_SERVER_CLIENT_STATUS_FAIL:
      return t('Failed');
    case PIFR_SERVER_CLIENT_STATUS_ENABLED:
      return t('Enabled');
    default:
      return 'Unknown';
  }
}

/**
 * Get the client status from the code.
 *
 * @param integer $client_environemnt Client environment code.
 * @return string Representation of client environment.
 */
function pifr_server_client_environment($client_environemnt) {
  switch ($client_environemnt) {
    case PIFR_SERVER_CLIENT_ENVIRONMENT_MYSQL_5_0_ISAM:
      return t('MySQL 5.0 ISAM');
    case PIFR_SERVER_CLIENT_ENVIRONMENT_POSTGRES:
      return t('PostgreSQL 8.3');
    case PIFR_SERVER_CLIENT_ENVIRONMENT_SQLITE:
      return t('SQLite');
    case PIFR_SERVER_CLIENT_ENVIRONMENT_MYSQL_5_1_ISAM:
      return t('MySQL 5.1 ISAM');
    case PIFR_SERVER_CLIENT_ENVIRONMENT_MYSQL_5_0_INNO:
      return t('MySQL 5.0 INNO');
    default:
      return 'Unknown';
  }
}

function pifr_server_client_step($type) {
  switch ($type) {
    case PIFR_SERVER_TEST_RESULT_SETUP:
      return t('Setup environment.');
    case PIFR_SERVER_TEST_RESULT_FETCH:
      return t('Detect invalid file URL.');
    case PIFR_SERVER_TEST_RESULT_CVS:
      return t('Detect invalid repository information.');
    case PIFR_SERVER_TEST_RESULT_PATCH:
      return t('Detect invalid patch format.');
    case PIFR_SERVER_TEST_RESULT_APPLY:
      return t('Detect non-applicable patch.');
    case PIFR_SERVER_TEST_RESULT_SYNTAX:
      return 'Detect invalid PHP syntax.';
    case PIFR_SERVER_TEST_RESULT_FORMAT:
      return '-- not implemented --';
    case PIFR_SERVER_TEST_RESULT_INSTALL:
      return 'Detect Drupal installation failure.';
    case PIFR_SERVER_TEST_RESULT_TEST:
      return 'Detect test run failure.';
    case PIFR_SERVER_TEST_RESULT_FAIL:
      return 'Detect failing test.';
    case PIFR_SERVER_TEST_RESULT_PASS:
      return 'Complete test suite with all tests passing.';
    default:
      return 'Unknown';
  }
}

class PIFRTestClient extends PIFRTest {

  protected $client;

  public function __construct(array $test) {
    parent::__construct($test);
    $this->client = pifr_server_client_get_test($test['test_id']);
  }

  public function getType() {
    return t('Client');
  }

  public function getTitle() {
    return t('Client #@client_id', array('@client_id' => $this->client['client_id']));
  }

  public function getInfo() {
    return $this->client;
  }

  public function getInfoTable() {
    $rows = array();

    $rows['type'] = array(t('Type'), pifr_server_client_type($this->client['type']));
    $rows['environment'] = array(t('Environment'), pifr_server_client_environment($this->client['environment_id']));
    $rows['status'] = array(t('Status'), pifr_server_client_status($this->client['status']));

    return $this->formatTable($rows);
  }
}
