<?php
// $Id: pifr_server.file.inc,v 1.20 2009/06/27 01:37:41 boombatower Exp $
/**
 * @file
 * Provide file functions.
 *
 * Copyright 2008-2009 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Get a file by ID.
 *
 * @param integer $file_id File ID.
 * @return mixed File or FALSE.
 */
function pifr_server_file_get($file_id) {
  return db_fetch_array(db_query('SELECT *
                                   FROM {pifr_file}
                                   WHERE file_id = %d', $file_id));
}

/**
 * Find a file by name and client.
 *
 * @param string $filename File name.
 * @param integer $branch_id Branch ID.
 * @return mixed File or FALSE.
 */
function pifr_server_file_find($file_name, $branch_id) {
  return db_fetch_array(db_query("SELECT *
                                   FROM {pifr_file}
                                   WHERE file_name LIKE '%s'
                                   AND branch_id = %d", $file_name, $branch_id));
}

function pifr_server_file_get_test($test_id) {
  return db_fetch_array(db_query('SELECT *
                                   FROM {pifr_file}
                                   WHERE test_id = %d', $test_id));
}

function pifr_server_file_branch_update($branch_id, $status_from, $status_to) {
  db_query('UPDATE {pifr_test}
            SET status = %d
            WHERE test_id IN (
              SELECT test_id
              FROM {pifr_file}
              WHERE branch_id = %d
            )
            AND status = %d', $status_to, $branch_id, $status_from);
}

function pifr_server_file_get_url(array $file) {
  $branch = pifr_server_branch_get($file['branch_id']);
  $project = pifr_server_project_get($branch['project_id']);
  $client = pifr_server_client_get($project['client_id']);

  return $file['file_url'];
}

function pifr_server_file_save(array $file, $check = TRUE) {
  if ($check) {
    // Check if branch already exists.
    $file_record = pifr_server_file_find($file['file_name'], $file['branch_id']);
    $file['file_id'] = ($file_record ? $file_record['file_id'] : NULL);
    $file['test_id'] = ($file_record ? $file_record['test_id'] : NULL);
  }

  if (isset($file['file_id'])) {
    // Existing project, update record.
    db_query("UPDATE {pifr_file}
              SET branch_id = %d,
                  issue_nid = %d,
                  issue_cid = %d,
                  file_url = '%s',
                  file_name = '%s',
                  test_id = %d
              WHERE file_id = %d", $file['branch_id'], $file['issue_nid'], $file['issue_cid'], $file['file_url'],
                                   $file['file_name'], $file['test_id'], $file['file_id']);
  }
  else {
    // Insert test record.
    $test = array(
      'type' => PIFR_SERVER_TEST_TYPE_FILE,
      'status' => PIFR_SERVER_TEST_STATUS_STUB,
      'client_id' => 0,
      'last_received' => time(),
      'last_requested' => 0,
      'last_tested' => 0,
      'test_count' => 0
    );
    $test = pifr_server_test_save($test);
    $file['test_id'] = $test['test_id'];

    // New project, insert record.
    db_query("INSERT INTO {pifr_file} (branch_id, issue_nid, issue_cid, file_url, file_name, test_id)
              VALUES (%d, %d, %d, '%s', '%s', %d)",
              $file['branch_id'], $file['issue_nid'], $file['issue_cid'],
              $file['file_url'], $file['file_name'], $file['test_id']);
    $file['file_id'] = db_last_insert_id('pifr_file', 'file_id');
  }
  return $file;
}

class PIFRTestFile extends PIFRTest {

  protected $file;

  public function __construct(array $test) {
    parent::__construct($test);
    $this->file = pifr_server_file_get_test($test['test_id']);
  }

  public function getType() {
    return t('File');
  }

  public function getTitle() {
    return check_plain($this->file['file_name']);
  }

  public function getInfo() {
    return $this->file;
  }

  public function getInfoTable() {
    $rows = array();

    $branch = PIFRTest::getInstanceById(PIFR_SERVER_TEST_TYPE_BRANCH, $this->file['branch_id']);

    $rows['file'] = array(t('File'), l($this->file['file_name'], pifr_server_file_get_url($this->file)));

    $branch_test = $branch->getTest();
    $rows['branch'] = array(t('Branch'), l($branch->getTitle(), 'pifr/test/' . $branch_test['test_id']));

    $branch_info = $branch->getInfo();
    $project = pifr_server_project_get($branch_info['project_id']);
    $client = pifr_server_client_get($project['client_id']);

    $rows['issue'] = array(t('Issue'), t('<a href="@node">#@nid</a>',
      array('@node' => "{$client['url']}node/{$this->file['issue_nid']}", '@nid' => $this->file['issue_nid'])));
    if ($this->file['issue_cid']) {
      $rows['issue'][1] .= t(' [<a href="@comment">comment #@cid</a>]',
        array('@comment' => "{$client['url']}node/{$this->file['issue_nid']}#comment-{$this->file['issue_cid']}",
               '@cid' => $this->file['issue_cid']));
    }

    return $this->formatTable($rows);
  }
}
