<?php
// $Id: pifr_server.log.inc,v 1.4 2009/03/13 03:07:27 boombatower Exp $
/**
 * @file
 * Provide event logging functions.
 *
 * Copyright 2008-2009 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * The pager ID, used to reference it.
 */
define('PIFR_SERVER_LOG_PAGER_ID', 0);

/*
 * Number of log items to display per page.
 */
define('PIFR_SERVER_LOG_PAGER_LARGE', 50);
define('PIFR_SERVER_LOG_PAGER_SMALL', 10);

/**
 * Log an event that occured to a test.
 *
 * @param interger $test_id Test ID.
 * @param string $message Message describing event.
 */
function pifr_server_log_event($test_id, $message) {
  db_query("INSERT INTO {pifr_log} (test_id, timestamp, message)
            VALUES (%d, %d, '%s')", $test_id, time(), $message);
}

/**
 * Get log of events.
 *
 * @param interger $test_id Test ID.
 * @return array List of events.
 */
function pifr_server_log_get($test_id = NULL) {
  if ($test_id) {
    $result = pager_query('SELECT *
                           FROM {pifr_log}
                           WHERE test_id = %d
                           ORDER BY timestamp DESC', PIFR_SERVER_LOG_PAGER_SMALL, PIFR_SERVER_LOG_PAGER_ID, NULL, $test_id);
  }
  else {
    $result = pager_query('SELECT *
                           FROM {pifr_log}
                           ORDER BY timestamp DESC', PIFR_SERVER_LOG_PAGER_LARGE, PIFR_SERVER_LOG_PAGER_ID);
  }

  $events = array();
  while ($event = db_fetch_array($result)) {
    $events[] = $event;
  }
  return $events;
}

/**
 * View event log. If test ID is specified then return events for the specified
 * test only.
 *
 * @param integer $test_id Test ID.
 * @return string HTML output.
 */
function pifr_server_log_view($test_id = NULL) {
  $events = pifr_server_log_get($test_id);

  $header = array(t('Timestamp'), t('Type'), t('Title'), t('Message'));
  if ($test_id) {
    unset($header[1]); // Type.
    unset($header[2]); // Title.
  }

  $rows = array();
  foreach ($events as $event) {
    $row = array();
    $row[] = format_date($event['timestamp'], 'small');

    // Add type and title fields if not on displaying test details.
    if (!$test_id) {
      $object = PIFRTest::getInstanceByTest($event['test_id']);

      $row[] = $object->getType();
      $row[] = l($object->getTitle(), 'pifr/test/' . $event['test_id']);
    }
    $row[] = $event['message'];

    $rows[] = $row;
  }

  if ($rows) {
    return theme('table', $header, $rows) .
            theme('pager', NULL, ($test_id ? PIFR_SERVER_LOG_PAGER_SMALL : PIFR_SERVER_LOG_PAGER_LARGE), PIFR_SERVER_LOG_PAGER_ID);
  }
  return '<p><em>' . t('No events to display.') . '</em></p>';
}
