<?php
// $Id: pifr_server.menu.inc,v 1.8 2009/04/17 05:56:49 boombatower Exp $
/**
 * @file
 * Provide menu items.
 *
 * Copyright 2008-2009 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Menu items for hook_menu().
 */
function pifr_server_menu_items() {
  $items = array();

  // General information: all users.
  $items['pifr/log'] = array(
    'title' => 'Event log',
    'page callback' => 'pifr_server_log_view',
    'access arguments' => array('access content'),
    'type' => MENU_CALLBACK,
    'file' => 'pifr_server.log.inc'
  );
  $items['pifr/status'] = array(
    'title' => 'Status',
    'page callback' => 'pifr_server_page_status',
    'access arguments' => array('access content'),
    'type' => MENU_CALLBACK,
    'file' => 'pifr_server.page.inc'
  );
  $items['pifr/statistics'] = array(
    'title' => 'Statistics',
    'page callback' => 'pifr_server_page_statistics',
    'access arguments' => array('access content'),
    'type' => MENU_CALLBACK,
    'file' => 'pifr_server.page.inc'
  );
  $items['pifr/test/%'] = array(
    'title' => 'View test',
    'page callback' => 'pifr_server_test_view',
    'page arguments' => array(2),
    'access arguments' => array('access content'),
    'type' => MENU_CALLBACK,
    'file' => 'pifr_server.test.inc'
  );

  // Client management: administrative.
  $items['admin/pifr/server'] = array(
    'title' => 'Server',
    'description' => 'Manage PIFR clients.',
    'page callback' => 'pifr_server_manage_dashboard',
    'access arguments' => array('pifr manage clients'),
    'file' => 'pifr_server.manage.inc',
  );
  $items['admin/pifr/server/dashboard'] = array(
    'title' => 'Dashboard',
    'type' => MENU_DEFAULT_LOCAL_TASK,
    'weight' => -10
  );
  $items['admin/pifr/server/edit/%'] = array(
    'title' => 'Edit client',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('pifr_server_manage_client_form', 4),
    'access arguments' => array('pifr manage clients'),
    'file' => 'pifr_server.manage.inc',
    'type' => MENU_CALLBACK,
  );
  $items['admin/pifr/server/enable/%'] = array(
    'title' => 'Request enable client',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('pifr_server_client_enable_confirm_form', 4),
    'access arguments' => array('pifr manage clients'),
    'file' => 'pifr_server.manage.inc',
    'type' => MENU_CALLBACK,
  );
  $items['admin/pifr/server/disable/%'] = array(
    'title' => 'Request enable client',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('pifr_server_client_disable_confirm_form', 4),
    'access arguments' => array('pifr manage clients'),
    'file' => 'pifr_server.manage.inc',
    'type' => MENU_CALLBACK,
  );
  $items['admin/pifr/server/remove/%'] = array(
    'title' => 'Remove client',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('pifr_server_client_remove_confirm_form', 4),
    'access arguments' => array('pifr manage clients'),
    'file' => 'pifr_server.manage.inc',
    'type' => MENU_CALLBACK,
  );

  // Client management: user specific.
  $items['user/%user/pifr'] = array(
    'title' => 'PIFR Clients',
    'page callback' => 'pifr_server_manage_dashboard',
    'page arguments' => array(1),
    'access arguments' => array('pifr manage own client'),
    'file' => 'pifr_server.manage.inc',
    'type' => MENU_LOCAL_TASK
  );
  $items['user/%user/pifr/dashboard'] = array(
    'title' => 'Dashboard',
    'type' => MENU_DEFAULT_LOCAL_TASK,
    'weight' => -10
  );
  $items['user/%user/pifr/add'] = array(
    'title' => 'Add client',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('pifr_server_manage_client_form', NULL, 1),
    'access callback' => 'pifr_server_access',
    'access arguments' => array('pifr add project client', 'pifr add test client'),
    'file' => 'pifr_server.manage.inc',
    'type' => MENU_LOCAL_TASK,
    'weight' => -9
  );
  $items['user/%user/pifr/edit/%'] = array(
    'title' => 'Edit client',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('pifr_server_manage_client_form', 4, 1),
    'access callback' => 'pifr_server_manage_access',
    'access arguments' => array(4, 1),
    'file' => 'pifr_server.manage.inc',
    'type' => MENU_CALLBACK,
  );
  $items['user/%user/pifr/enable/%'] = array(
    'title' => 'Request enable client',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('pifr_server_client_enable_confirm_form', 4, 1),
    'access callback' => 'pifr_server_manage_access',
    'access arguments' => array(4, 1),
    'file' => 'pifr_server.manage.inc',
    'type' => MENU_CALLBACK,
  );
  $items['user/%user/pifr/disable/%'] = array(
    'title' => 'Disable client',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('pifr_server_client_disable_confirm_form', 4, 1),
    'access callback' => 'pifr_server_manage_access',
    'access arguments' => array(4, 1),
    'file' => 'pifr_server.manage.inc',
    'type' => MENU_CALLBACK,
  );
  $items['user/%user/pifr/remove/%'] = array(
    'title' => 'Remove client',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('pifr_server_client_remove_confirm_form', 4, 1),
    'access callback' => 'pifr_server_manage_access',
    'access arguments' => array(4, 1),
    'file' => 'pifr_server.manage.inc',
    'type' => MENU_CALLBACK,
  );

  return $items;
}
