<?php
// $Id: pifr_server.page.inc,v 1.7 2009/06/11 20:39:36 boombatower Exp $
/**
 * @file
 * Provide general access pages.
 *
 * Copyright 2008-2009 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * View the status of testing.
 *
 * @return string HTML output.
 */
function pifr_server_page_status() {
  drupal_add_css(drupal_get_path('module', 'pifr') . '/pifr.css');

  $form = array();

  $form['general'] = array(
    '#type' => 'fieldset',
    '#title' => t('General'),
    '#weight' => -10,
    '#value' => t('Testing is %status. Please see home page for details.',
                  array('%status' => (PIFR_ACTIVE ? 'enabled': 'disabled'))),
  );

  $form['statistics'] = array(
    '#type' => 'fieldset',
    '#title' => t('Statistics'),
    '#weight' => -9,
    '#value' => pifr_server_page_status_statistics(),
  );

  $form['project_clients'] = array(
    '#type' => 'fieldset',
    '#title' => t('Project clients'),
    '#value' => pifr_server_page_status_client_list(PIFR_SERVER_CLIENT_TYPE_PROJECT),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#weight' => -8,
  );

  $form['test_clients'] = array(
    '#type' => 'fieldset',
    '#title' => t('Test clients'),
    '#value' => pifr_server_page_status_client_list(PIFR_SERVER_CLIENT_TYPE_TEST),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#weight' => -7,
  );

  return drupal_render($form);
}

function pifr_server_page_status_statistics() {
  // Queue.
  $queue = array(
    '#chart_id' => 'status_queue',
    '#title' => t('Queue'),
    '#type' => CHART_TYPE_PIE_3D,
    '#size' => chart_size(350, 150),
    '#data' => array(),
    '#labels' => array(),
  );

  $result = db_query('SELECT COUNT(test_id) AS count, status
                      FROM {pifr_test}
                      GROUP BY status
                      ORDER BY count');

  while ($type = db_fetch_array($result)) {
    $queue['#data'][$type['status']] = $type['count'];
    $queue['#labels'][$type['status']] = pifr_server_test_status($type['status']) . ': ' . $type['count'];
  }

  $out = chart_render($queue);

  // Capacity.
  $active = 0;
  $total = 0;

  $clients = pifr_server_client_get_type(PIFR_SERVER_CLIENT_TYPE_TEST);
  foreach ($clients as $client) {
    if ($client['status'] == PIFR_SERVER_CLIENT_STATUS_DISABLED) {
      continue;
    }

    $deadtime = time() - $client['last_active'];
    if ($deadtime < PIFR_SERVER_CLIENT_TEST_INTERVAL * 60 * 60) {
      $active++;
    }
    $total++;
  }

  $capacity = array(
    '#chart_id' => 'capacity',
    '#title' => t('Capacity'),
    '#type' => CHART_TYPE_GMETER,
    '#size' => chart_size(300, 150),
    '#data' => array($total ? $active / $total * 100 : 0),
  );
  $out .= chart_render($capacity);

  // Usage.
  $sent = isset($queue['#data'][PIFR_SERVER_TEST_STATUS_SENT]) ? $queue['#data'][PIFR_SERVER_TEST_STATUS_SENT] : 0;

  $usage = array(
    '#chart_id' => 'usage',
    '#title' => t('Usage'),
    '#type' => CHART_TYPE_GMETER,
    '#size' => chart_size(300, 150),
    '#data' => array($actve ? $sent / $active * 100 : 0),
    '#data_colors' => array('00FF00', 'FFFF00', 'FF8040', 'FF0000'),
  );
  $out .= chart_render($usage);

  return $out;
}

function pifr_server_page_status_client_list($client_type) {
  $clients = pifr_server_client_get_type($client_type);

  $header = array(t('Client'), t('Environment'), t('Status'), t('Last active'), t('Manager'));
  if ($client_type == PIFR_SERVER_CLIENT_TYPE_PROJECT) {
    unset($header[1]);
  }
  $rows = array();
  foreach ($clients as $client) {
    if ($client['status'] == PIFR_SERVER_CLIENT_STATUS_DISABLED) {
      continue;
    }

    $row = array();

    $row[] = $client['client_id'];

    if ($client_type == PIFR_SERVER_CLIENT_TYPE_TEST) {
      $row[] = pifr_server_client_environment($client['environment_id']);
    }

    $row[] = pifr_server_client_status($client['status']);
    $row[] = pifr_server_format_date($client['last_active']);
    $row[] = theme('username', user_load($client['uid']));

    $deadtime = time() - $client['last_active'];
    if ($deadtime < 60 * 60 * 2) {
      $class = 'pifr-pass';
    }
    elseif ($deadtime > 60 * 60 * 2 && $deadtime < 60 * 60 * 24) {
      $class = 'pifr-exception';
    }
    else {
      $class = 'pifr-fail';
    }

    $rows[] = array('class' => $class, 'data' => $row);
  }
  return theme('table', $header, $rows);
}

function pifr_server_page_statistics() {
  $out = '';

  // Test result breakdown graph.
  $results = db_query('SELECT code, COUNT(code) AS count
                       FROM {pifr_result}
                       WHERE test_id NOT IN (
                         SELECT test_id
                         FROM {pifr_client}
                       )
                       GROUP BY code
                       ORDER BY count');

  $chart = array(
    '#chart_id' => 'result_breakdown',
    '#title' => t('Test result breakdown'),
    '#type' => CHART_TYPE_PIE_3D,
    '#size' => chart_size(600, 200)
  );

  $chart['#data'] = array();
  $chart['#labels'] = array();
  while ($result = db_fetch_object($results)) {
    $chart['#data'][] = $result->count;
    $chart['#labels'][] = pifr_server_result_get_summary_message_minimal($result->code);
  }

  $out .= chart_render($chart);

  // Numerical test statistics.
  $test_count = db_result(db_query('SELECT COUNT(test_id)
                                    FROM {pifr_test}
                                    WHERE test_count > 0'));
  $suite_runs = db_result(db_query('SELECT SUM(test_count)
                                    FROM {pifr_test}'));
  $assertions = db_result(db_query('SELECT MAX(pass + fail + exception)
                                    FROM {pifr_result}'));

  $header = array(t('Key'), t('Value'));
  $rows = array();
  $rows[] = array(t('Test count'), $test_count);
  $rows[] = array(t('Test suite runs'), $suite_runs);
  $rows[] = array(t('Maximum assertion count'), $assertions);

  $out .= theme('table', $header, $rows, array('style' => 'width: auto'));

  return $out;
}
