<?php
// $Id: pifr_server.test.inc,v 1.29 2009/06/18 23:05:11 boombatower Exp $
/**
 * @file
 * Provide client test functions to confirm that a client is funcitoning properly.
 *
 * Copyright 2008-2009 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

function pifr_server_test_get($test_id) {
  return db_fetch_array(db_query('SELECT *
                                   FROM {pifr_test}
                                   WHERE test_id = %d', $test_id));
}

function pifr_server_test_get_client($client_id) {
  return db_fetch_array(db_query('SELECT *
                                   FROM {pifr_test}
                                   WHERE client_id = %d
                                   AND status = %d', $client_id, PIFR_SERVER_TEST_STATUS_SENT));
}

function pifr_server_test_get_next(array $client) {
  $test = pifr_server_test_get($client['test_id']);
  if (time() - $test['last_tested'] >= (PIFR_SERVER_CLIENT_TEST_INTERVAL * 60 * 60)) {
    // Client needs to be re-confirmed.
    pifr_server_client_confirmation_init($client);
    $client = pifr_server_client_get($client['client_id']);
  }

  if ($client['status'] == PIFR_SERVER_CLIENT_STATUS_TESTING) {
    // Next client test file.
    $test = pifr_server_test_get($client['test_id']);
    pifr_server_test_sent($test, $client);
    return pifr_server_client_confirmation_next($client);
  }
  else if ($client['status'] == PIFR_SERVER_CLIENT_STATUS_ENABLED) {
    // If the client currently has a test out, reset the test.
    $test = pifr_server_test_get_client($client['client_id']);
    if ($test) {
      $test['status'] = PIFR_SERVER_TEST_STATUS_QUEUED;
      pifr_server_test_save($test);
      pifr_server_log_event($test['test_id'], t('Test reset by client request.'));
    }

    // Next branch or file test.
    $test = db_fetch_array(db_query('SELECT t.*
                                     FROM {pifr_test} t
                                     LEFT JOIN {pifr_result} r
                                       ON (t.test_id = r.test_id AND r.type = %d)
                                     WHERE t.type != %d
                                     AND t.status = %d
                                     AND r.result_id IS NULL
                                     ORDER BY t.type, t.test_count, t.last_tested, t.test_id
                                     LIMIT 1',
                                     $client['environment_id'], PIFR_SERVER_TEST_TYPE_CLIENT, PIFR_SERVER_TEST_STATUS_QUEUED));
    if ($test) {
      pifr_server_test_sent($test, $client);
      return pifr_server_test_xmlrpc($test);
    }
    return array();
  }
}

function pifr_server_test_get_since($timestamp) {
  $result = db_query('SELECT *
                      FROM {pifr_test}
                      WHERE status = %d
                      AND last_tested >= %d', PIFR_SERVER_TEST_STATUS_RESULT, $timestamp);
  $tests = array();
  while ($test = db_fetch_array($result)) {
    $tests[] = $test;
  }
  return $tests;
}

function pifr_server_test_xmlrpc(array $test) {
  $response = array(
    'test_id' => $test['test_id'],
  );

  if ($test['type'] == PIFR_SERVER_TEST_TYPE_FILE) {
    $file = pifr_server_file_get_test($test['test_id']);
    $response['file_url'] = pifr_server_file_get_url($file);

    $branch = pifr_server_branch_get($file['branch_id']);
    $project = pifr_server_project_get($branch['project_id']);
  }
  else {
    $branch = pifr_server_branch_get_test($test['test_id']);
    $project = pifr_server_project_get($branch['project_id']);
  }

  if ($branch['core_branch_id']) {
    // Load core branch.
    $core_branch = pifr_server_branch_get($branch['core_branch_id']);
    $core_project = pifr_server_project_get($core_branch['project_id']);

    $response['module'] = array(
      'repository' => array(
        'type' => $project['repository_type'],
        'url' => $project['repository_url'],
      ),
      'branch_identifier' => $branch['identifier'],
    );
  }
  else {
    $core_branch = $branch;
    $core_project = $project;
  }

  $response['core'] = array(
    'repository' => array(
      'type' => $core_project['repository_type'],
      'url' => $core_project['repository_url'],
    ),
    'branch_identifier' => $core_branch['identifier'],
  );
  return $response;
}

function pifr_server_test_queue(array $test) {
  if ($test['status'] == PIFR_SERVER_TEST_STATUS_SENT) {
    pifr_server_log_event($test['test_id'], t('Re-test request ignored (test is active).'));
  }
  else {
    $test['status'] = PIFR_SERVER_TEST_STATUS_QUEUED;
    $test['last_received'] = time();
    pifr_server_test_save($test);
    pifr_server_result_clear($test['test_id']);
    pifr_server_log_event($test['test_id'], t('@action request received.',
                          array('@action' => $test['test_count'] == 0 ? t('Test') : t('Re-test'))));
  }
}

function pifr_server_test_sent(array $test, array $client) {
  $test['status'] = PIFR_SERVER_TEST_STATUS_SENT;
  $test['client_id'] = $client['client_id'];
  $test['last_requested'] = time();
  pifr_server_test_save($test);

  pifr_server_log_event($test['test_id'], t('Requested by test client #@client_id.',
                                            array('@client_id' => $client['client_id'])));
}

function pifr_server_test_result(array $test, array $result, array $client) {
  $results = pifr_server_result_get_all($test['test_id']);

  // Update test record.
  if ($result['code'] != PIFR_SERVER_TEST_RESULT_PASS || count($results) == PIFR_SERVER_CLIENT_ENVIRONMENT_COUNT) {
    $test['status'] = PIFR_SERVER_TEST_STATUS_RESULT;
  }
  else {
    $test['status'] = PIFR_SERVER_TEST_STATUS_QUEUED;
  }
  $test['last_tested'] = time();
  $test['test_count']++;
  pifr_server_test_save($test);

  // Save test results.
  if ($client['status'] == PIFR_SERVER_CLIENT_STATUS_TESTING) {
    $last = pifr_server_client_confirmation_last($client);
    $result['type'] = $last + 1;
  }
  else {
    $result['type'] = $client['environment_id'];
  }
  $result['details'] = serialize($result['details']);
  pifr_server_result_save($result);

  pifr_server_log_event($test['test_id'], t('Result received from test client #@client_id.',
                                            array('@client_id' => $client['client_id'])));

  if ($client['status'] == PIFR_SERVER_CLIENT_STATUS_TESTING) {
    pifr_server_client_confirmation_check($client, $result);
  }

  if ($test['type'] == PIFR_SERVER_TEST_TYPE_BRANCH) {
    // Branch result may effect related files.
    $branch = pifr_server_branch_get_test($test['test_id']);
    if ($result['code'] == PIFR_SERVER_TEST_RESULT_PASS) {
      // Queue all tests that have been postponed.
      pifr_server_file_branch_update($branch['branch_id'], PIFR_SERVER_TEST_STATUS_POSTPONED, PIFR_SERVER_TEST_STATUS_QUEUED);
    }
    else {
      // Postpone all tests that have been queued.
      pifr_server_file_branch_update($branch['branch_id'], PIFR_SERVER_TEST_STATUS_QUEUED, PIFR_SERVER_TEST_STATUS_POSTPONED);
    }
  }
}

function pifr_server_test_save(array $test) {
  if (isset($test['test_id'])) {
    // Existing test, update record.
    db_query('UPDATE {pifr_test}
              SET type = %d,
                  status = %d,
                  client_id = %d,
                  last_received = %d,
                  last_requested = %d,
                  last_tested = %d,
                  test_count = %d
              WHERE test_id = %d', $test['type'], $test['status'], $test['client_id'], $test['last_received'],
                                   $test['last_requested'], $test['last_tested'], $test['test_count'], $test['test_id']);
  }
  else {
    // New test, insert record.
    db_query("INSERT INTO {pifr_test} (type, status, client_id, last_received, last_requested, last_tested, test_count)
              VALUES (%d, %d, %d, %d, %d, %d, %d)",
              $test['type'], $test['status'], $test['client_id'], $test['last_received'],
              $test['last_requested'], $test['last_tested'], $test['test_count']);
    $test['test_id'] = db_last_insert_id('pifr_test', 'test_id');
  }
  return $test;
}

function pifr_server_test_remove($test_id) {
  db_query('DELETE FROM {pifr_test}
            WHERE test_id = %d', $test_id);
}

function pifr_server_test_view($test_id) {
  drupal_add_css(drupal_get_path('module', 'pifr') . '/pifr.css');
  drupal_add_css(drupal_get_path('module', 'pifr_server') . '/pifr_server.css');

  $test = pifr_server_test_get($test_id);
  if ($test) {
    $object = PIFRTest::getInstanceByTest($test_id);

    drupal_set_title(t('View test - @title', array('@title' => $object->getTitle())));

    $form = array();

    $form['status'] = array(
      '#type' => 'fieldset',
      '#title' => t('Test status'),
      '#attributes' => array('id' => 'pifr-status', 'class' => 'pifr-test'),
      '#weight' => -10,
    );
    $form['status']['table'] = array(
      '#type' => 'markup',
      '#value' => theme('table', array(), $object->getTestTable()),
    );

    $form['info'] = array(
      '#type' => 'fieldset',
      '#title' => t('@type information', array('@type' => $object->getType())),
      '#attributes' => array('id' => 'pifr-info', 'class' => 'pifr-test'),
      '#weight' => -9,
    );
    $form['info']['table'] = array(
      '#type' => 'markup',
      '#value' => theme('table', array(), $object->getInfoTable()),
    );

    $form['log'] = array(
      '#type' => 'fieldset',
      '#title' => t('Event log'),
      '#attributes' => array('id' => 'pifr-log'),
      '#collapsible' => TRUE,
      '#collapsed' => !(isset($_GET['page']) ||
                       (isset($_SERVER['HTTP_REFERER']) && strpos($_SERVER['HTTP_REFERER'], '?page') !== FALSE)),
      '#weight' => -8,
    );
    $form['log']['table'] = array(
      '#type' => 'markup',
      '#value' => pifr_server_log_view($test_id),
    );

    $form['result'] = array(
      '#type' => 'fieldset',
      '#title' => t('Test results'),
      '#attributes' => array('id' => 'pifr-result'),
      '#weight' => -7,
    );

    $map = array(
      'pass' => theme('image', 'misc/watchdog-ok.png'),
      'fail' => theme('image', 'misc/watchdog-error.png'),
      'exception' => theme('image', 'misc/watchdog-warning.png'),
    );

    if ($test['type'] == PIFR_SERVER_TEST_TYPE_CLIENT) {
      $header = array(t('Status'), t('Step'));
      $rows = array();

      for ($i = 1; $i <= PIFR_SERVER_TEST_RESULT_PASS; $i++) {
        if ($i == PIFR_SERVER_TEST_RESULT_FORMAT) {
          continue;
        }

        $row = array();
        $row['status'] = $map['exception'];
        $row['step'] = pifr_server_client_step($i);

        $rows[$i] = $row;
      }

      $results = pifr_server_result_get_all($test_id);
      foreach ($results as $type => $result) {
        $row = &$rows[$type];
        $pass = $result['code'] == $type;
        $row['status'] = $pass ? $map['pass'] : $map['fail'];

        if (!$pass) {
          $row['step'] .= ' [reason: ' . pifr_server_result_get_summary_message($result['code']) . ']';
        }
      }

      if (isset($results[PIFR_SERVER_TEST_RESULT_FETCH])) {
        $pass = $results[PIFR_SERVER_TEST_RESULT_FETCH]['code'] == PIFR_SERVER_TEST_RESULT_FETCH;
        $rows[PIFR_SERVER_TEST_RESULT_SETUP]['status'] = $pass ? $map['pass'] : $map['fail'];
      }

      $form['result']['table'] = array(
        '#type' => 'markup',
        '#value' => theme('table', $header, $rows),
      );
    }
    else {
      $form['result']['summary'] = array(
        '#type' => 'markup',
        '#value' => pifr_server_result_get_summary($test_id, TRUE),
        '#attributes' => array('id' => 'pifr-summary'),
      );

      $header = array(t('Message'), t('Group'), t('Filename'), t('Line'), t('Function'), t('Status'));
      $results = pifr_server_result_get_all($test_id);
      foreach ($results as $type => $result) {
        if ($result['code'] != PIFR_SERVER_TEST_RESULT_PASS) {
          foreach ($result['result_details'] as $detail) {
            $rows = array();
            foreach ($detail['assertions'] as $assertion) {
              $row = array();
              $row[] = $assertion['message'];
              $row[] = $assertion['message_group'];
              $row[] = $assertion['file'];
              $row[] = $assertion['line'];
              $row[] = $assertion['function'];
              $row[] = $map[$assertion['status']];

              $rows[] = array('class' => 'pifr-' . $assertion['status'], 'data' => $row);
            }

            $form['result'][] = array(
              '#type' => 'fieldset',
              '#title' => $detail['test_name'],
              '#value' => theme('table', $header, $rows),
            );
          }
        }
      }
    }

    return drupal_render($form);
  }

  drupal_set_message(t('Invalid test.'), 'error');
  return '';
}

function pifr_server_test_type($test_type) {
  switch ($test_type) {
    case PIFR_SERVER_TEST_TYPE_CLIENT:
      return t('Client');
    case PIFR_SERVER_TEST_TYPE_BRANCH:
      return t('Branch');
    case PIFR_SERVER_TEST_TYPE_FILE:
      return t('File');
  }
}

function pifr_server_test_status($status) {
  switch ($status) {
    case PIFR_SERVER_TEST_STATUS_STUB:
      return t('Stub');
    case PIFR_SERVER_TEST_STATUS_QUEUED:
      return t('Queued');
    case PIFR_SERVER_TEST_STATUS_POSTPONED:
      return t('Postponed');
    case PIFR_SERVER_TEST_STATUS_SENT:
      return t('Sent');
    case PIFR_SERVER_TEST_STATUS_RESULT:
      return t('Result');
  }
}

function pifr_server_test_title($test_type, array $object) {
  switch ($test_type) {
    case PIFR_SERVER_TEST_TYPE_CLIENT:
      return t('Client #@id', array('@id' => $object['client_id']));
    case PIFR_SERVER_TEST_TYPE_BRANCH:
      return t('@identifier branch', array('@identifier' => $object['identifier']));
    case PIFR_SERVER_TEST_TYPE_FILE:
      return t('@file', array('@file' => $object['file_name']));
  }
}

abstract class PIFRTest {

  protected $test;

  protected function __construct(array $test) {
    $this->test = $test;
  }

  public static function getInstanceByTest($test_id) {
    $test = pifr_server_test_get($test_id);

    switch ($test['type']) {
      case PIFR_SERVER_TEST_TYPE_CLIENT:
        return new PIFRTestClient($test);
      case PIFR_SERVER_TEST_TYPE_BRANCH:
        return new PIFRTestBranch($test);
      case PIFR_SERVER_TEST_TYPE_FILE:
        return new PIFRTestFile($test);
    }
    return FALSE;
  }

  public static function getInstanceById($type, $object_id) {
    switch ($type) {
      case PIFR_SERVER_TEST_TYPE_CLIENT:
        $object = pifr_server_client_get($object_id);
        break;
      case PIFR_SERVER_TEST_TYPE_BRANCH:
        $object = pifr_server_branch_get($object_id);
        break;
      case PIFR_SERVER_TEST_TYPE_FILE:
        $object = pifr_server_file_get($object_id);
        break;
    }
    if ($object) {
      return self::getInstanceByTest($object['test_id']);
    }
    return FALSE;
  }

  public function getTest() {
    return $this->test;
  }

  public function getTestTable() {
    $rows = array();

    $rows['status'] = array(t('Status'), pifr_server_test_status($this->test['status']));
    $rows['last_received'] = array(t('Last received'), pifr_server_format_date($this->test['last_received']));
    $rows['last_requested'] = array(t('Last requested'), pifr_server_format_date($this->test['last_requested']));
    $rows['last_tested'] = array(t('Last tested'), pifr_server_format_date($this->test['last_tested']));
    $rows['test_count'] = array(t('Test count'), $this->test['test_count']);

    return $this->formatTable($rows);
  }

  public abstract function getType();

  public abstract function getTitle();

  public abstract function getInfo();

  public abstract function getInfoTable();

  protected function formatTable(array $rows) {
    foreach ($rows as &$row) {
      $row[0] = array('header' => TRUE, 'data' => $row[0]);
    }
    return $rows;
  }
}
